@@ -12,8 +12,11 @@ use Predis\Client as Redis;
 
 class IcingaRedis
 {
-    /** @var static The singleton */
-    protected static $instance;
+    public const DEFAULT_HOST = 'localhost';
+
+    public const DEFAULT_PORT = 6380;
+
+    public const DEFAULT_DATABASE = 0;
 
     /** @var Redis Connection to the Icinga Redis */
     private $redis;
@@ -24,15 +27,12 @@ class IcingaRedis
     /**
      * Get the singleton
      *
+     * @deprecated Use {@see Backend::getRedis()} instead
      * @return static
      */
     public static function instance(): self
     {
-        if (self::$instance === null) {
-            self::$instance = new static();
-        }
-
-        return self::$instance;
+        return Backend::getRedis();
     }
 
     /**
@@ -40,19 +40,17 @@ class IcingaRedis
      *
      * @return bool
      */
-    public static function isUnavailable(): bool
+    public function isUnavailable(): bool
     {
-        $self = self::instance();
-
-        if (! $self->redisUnavailable && $self->redis === null) {
+        if (! $this->redisUnavailable && $this->redis === null) {
             try {
-                $self->getConnection();
+                $this->getConnection();
             } catch (Exception $_) {
                 // getConnection already logs the error
             }
         }
 
-        return $self->redisUnavailable;
+        return $this->redisUnavailable;
     }
 
     /**
@@ -126,7 +124,7 @@ class IcingaRedis
      */
     public static function fetchHostState(array $ids, array $columns): Generator
     {
-        return self::fetchState('icinga:host:state', $ids, $columns);
+        return Backend::getRedis()->fetchState('icinga:host:state', $ids, $columns);
     }
 
     /**
@@ -139,7 +137,7 @@ class IcingaRedis
      */
     public static function fetchServiceState(array $ids, array $columns): Generator
     {
-        return self::fetchState('icinga:service:state', $ids, $columns);
+        return Backend::getRedis()->fetchState('icinga:service:state', $ids, $columns);
     }
 
     /**
@@ -151,10 +149,10 @@ class IcingaRedis
      *
      * @return Generator
      */
-    protected static function fetchState(string $key, array $ids, array $columns): Generator
+    protected function fetchState(string $key, array $ids, array $columns): Generator
     {
         try {
-            $results = self::instance()->getConnection()->hmget($key, $ids);
+            $results = $this->getConnection()->hmget($key, $ids);
         } catch (Exception $_) {
             // The error has already been logged elsewhere
             return;
@@ -166,17 +164,6 @@ class IcingaRedis
                 $keyMap = array_fill_keys($columns, null);
                 unset($keyMap['is_overdue']); // Is calculated by Icinga DB, not Icinga 2, hence it's never in redis
 
-                // TODO: Remove once https://github.com/Icinga/icinga2/issues/9427 is fixed
-                $data['state_type'] = $data['state_type'] === 0 ? 'soft' : 'hard';
-
-                if (isset($data['in_downtime']) && is_bool($data['in_downtime'])) {
-                    $data['in_downtime'] = $data['in_downtime'] ? 'y' : 'n';
-                }
-
-                if (isset($data['is_acknowledged']) && is_int($data['is_acknowledged'])) {
-                    $data['is_acknowledged'] = $data['is_acknowledged'] ? 'y' : 'n';
-                }
-
                 yield $ids[$i] => array_intersect_key(array_merge($keyMap, $data), $keyMap);
             }
         }
@@ -192,28 +179,18 @@ class IcingaRedis
     public static function getLastIcingaHeartbeat(Redis $redis = null)
     {
         if ($redis === null) {
-            $redis = self::instance()->getConnection();
+            $redis = Backend::getRedis()->getConnection();
         }
 
-        // Predis doesn't support streams (yet).
-        // https://github.com/predis/predis/issues/607#event-3640855190
-        $rs = $redis->executeRaw(['XREAD', 'COUNT', '1', 'STREAMS', 'icinga:stats', '0']);
+        $stream = 'icinga:stats';
 
-        if (! is_array($rs)) {
-            return null;
-        }
+        $rs = $redis->xread(1, null, [$stream], '0');
 
-        $key = null;
+        if (is_array($rs)) {
+            $timestampKeyPos = array_search('timestamp', $rs[$stream][0][1], true);
 
-        foreach ($rs[0][1][0][1] as $kv) {
-            if ($key === null) {
-                $key = $kv;
-            } else {
-                if ($key === 'timestamp') {
-                    return $kv / 1000;
-                }
-
-                $key = null;
+            if ($timestampKeyPos !== false && isset($rs[$stream][0][1][$timestampKeyPos + 1])) {
+                return $rs[$stream][0][1][$timestampKeyPos + 1] / 1000;
             }
         }
 
@@ -241,9 +218,11 @@ class IcingaRedis
         $section = $redisConfig->getSection('redis1');
 
         $redis = new Redis([
-            'host'      => $section->get('host', 'localhost'),
-            'port'      => $section->get('port', 6380),
-            'password'  => $section->get('password', ''),
+            'host'      => $section->get('host', static::DEFAULT_HOST),
+            'port'      => $section->get('port', static::DEFAULT_PORT),
+            'database'  => $section->get('database', static::DEFAULT_DATABASE),
+            'username'  => $section->get('username'),
+            'password'  => $section->get('password'),
             'timeout'   => 0.5
         ] + self::getTlsParams($moduleConfig));
 
@@ -279,8 +258,10 @@ class IcingaRedis
 
         $redis = new Redis([
             'host'      => $host,
-            'port'      => $section->get('port', 6380),
-            'password'  => $section->get('password', ''),
+            'port'      => $section->get('port', static::DEFAULT_PORT),
+            'database'  => $section->get('database', static::DEFAULT_DATABASE),
+            'username'  => $section->get('username'),
+            'password'  => $section->get('password'),
             'timeout'   => 0.5
         ] + self::getTlsParams($moduleConfig));
 
