@@ -7,7 +7,7 @@ namespace Icinga\Module\Icingadb\Model;
 use Icinga\Module\Icingadb\Model\Behavior\ReRoute;
 use ipl\Orm\Behavior\Binary;
 use ipl\Orm\Behaviors;
-use ipl\Orm\Model;
+use Icinga\Module\Icingadb\Common\Model;
 use ipl\Orm\Relations;
 
 /**
@@ -91,7 +91,8 @@ class Hostgroup extends Model
         $relations->belongsTo('zone', Zone::class);
 
         $relations->belongsToMany('customvar', Customvar::class)
-            ->through(HostgroupCustomvar::class);
+            ->through(HostgroupCustomvar::class)
+            ->setThroughAlias('t_hostgroup_customvar');
         $relations->belongsToMany('customvar_flat', CustomvarFlat::class)
             ->through(HostgroupCustomvar::class);
         $relations->belongsToMany('vars', Vars::class)
